/// -----------------ROS2 Client node of AMC4030 controller---------------///
/// Author: Alvaro Vazquez Ramos------------------------------------------///
/// Company: ASTIBOT -----------------------------------------------------///
/// Description: program for creation of ROS2 client node for AMC4030-----///
/// controller program.---------------------------------------------------///
/// The calling for creating the client node must include the desired ----///
/// positon. Example of node creation: -----------------------------------///
/// <ros2 run cpp_srvcli client 50 50 0 20> The desired position is ------///
/// {50,50,0} mm with the speed 20 mm/s ----------------------------------///

#include "rclcpp/rclcpp.hpp"
#include "controller_interfaces/srv/movement.hpp"

#include <chrono>
#include <cstdlib>
#include <memory>

using namespace std::chrono_literals;

int main(int argc, char **argv)
{
  rclcpp::init(argc, argv);

  if (argc != 5) {
      RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "usage: movement_client x y z speed");
      return 1;
  }

  std::shared_ptr<rclcpp::Node> node = rclcpp::Node::make_shared("movement_client");
  rclcpp::Client<controller_interfaces::srv::Movement>::SharedPtr client =
    node->create_client<controller_interfaces::srv::Movement>("movement");

  auto request = std::make_shared<controller_interfaces::srv::Movement::Request>();
  request->x = atoll(argv[1]);
  request->y = atoll(argv[2]);
  request->z = atoll(argv[3]);
  request->speed = atoll(argv[4]);

  while (!client->wait_for_service(1s)) {
    if (!rclcpp::ok()) {
      RCLCPP_ERROR(rclcpp::get_logger("rclcpp"), "Interrupted while waiting for the service. Exiting.");
      return 0;
    }
    RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "service not available, waiting again...");
  }

  auto result = client->async_send_request(request);
  // Wait for the result.
  if (rclcpp::spin_until_future_complete(node, result) ==
    rclcpp::executor::FutureReturnCode::SUCCESS)
  {
	RCLCPP_INFO(rclcpp::get_logger("rclcpp"), result.get()->resp);
  } 
  else {
    RCLCPP_ERROR(rclcpp::get_logger("rclcpp"), "Failed to call service movement");
  }

  rclcpp::shutdown();
  return 0;
}