/// -----------------ROS2 Server node of AMC4030 controller---------------///
/// Author: Alvaro Vazquez Ramos------------------------------------------///
/// Company: ASTIBOT -----------------------------------------------------///
/// Description: program for creation of ROS2 server node for AMC4030 ----///
///controller program.----------------------------------------------------///
/// Node parameters:------------------------------------------------------///
/// - int64_t COM_TYPE: communication type with the controller. It can be ///
/// 0, 1, 2: LAN, USB, UART.----------------------------------------------///
/// - int64_t port_id: identificador del puerto.--------------------------///
/// - int64_t n_baud: baud rate of the transmission.----------------------///
/// For the movement function it uses de Controller_AMC4030.exe program,  ///
/// The arguments for calling the program is x y z speed.-----------------///


#include "rclcpp/rclcpp.hpp"
#include "controller_interfaces/srv/movement.hpp"

#include <windows.h>
#include <direct.h> //_getcwd
#include <stdio.h>
#include <tchar.h>
#include <memory>
#include <iostream>


int64_t com_type;
int64_t port_id;
int64_t n_baud;

void move(const std::shared_ptr<controller_interfaces::srv::Movement::Request> request,
          std::shared_ptr<controller_interfaces::srv::Movement::Response>      response)
{
  RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "Incoming request\nx: %ld" " y: %ld" " z: %ld" " speed: %ld",
                request->x, request->y, request->z, request->speed);
  char* package_path;
  if ((package_path = _getcwd(NULL,0)) == NULL) {
	  std::cerr << "Error calling getcwd\n";
	  exit(EXIT_FAILURE);
  }
  std::string command = std::string(package_path)+"\\Controlador_AMC4030.exe "+
	std::to_string(com_type)+" "+
	std::to_string(port_id) +" "+
	std::to_string(n_baud)  +" "+
	std::to_string(request->x)  +" "+
	std::to_string(request->y)  +" "+
	std::to_string(request->z)  +" "+
	std::to_string(request->speed);
  int cont_code = system(command.c_str());
  
  response->resp = (cont_code == 0) ? "Movement successfull" : "Movement failed";
  RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "sending back response: [%s]", response->resp.c_str());
}

int main(int argc, char **argv)
{
  rclcpp::init(argc, argv);

  std::shared_ptr<rclcpp::Node> node = rclcpp::Node::make_shared("movement_server");
  
  com_type = node->declare_parameter<int64_t>("COM_TYPE",2);
  port_id = node->declare_parameter<int64_t>("Port_id",24);
  n_baud = node->declare_parameter<int64_t>("Baud_rate",115200);
  
  rclcpp::Service<controller_interfaces::srv::Movement>::SharedPtr service =
    node->create_service<controller_interfaces::srv::Movement>("movement", &move);

  RCLCPP_INFO(rclcpp::get_logger("rclcpp"), "Ready to receive position.");

  rclcpp::spin(node);
  rclcpp::shutdown();
}